@@ -11,6 +11,17 @@ use Scalar::Util ();
 
 use Text::Xslate::PP::Const;
 
+use constant TXframe_NAME       => Text::Xslate::PP::TXframe_NAME;
+use constant TXframe_OUTPUT     => Text::Xslate::PP::TXframe_OUTPUT;
+use constant TXframe_RETADDR    => Text::Xslate::PP::TXframe_RETADDR;
+use constant TXframe_START_LVAR => Text::Xslate::PP::TXframe_START_LVAR;
+
+use constant TX_VERBOSE_DEFAULT => Text::Xslate::PP::TX_VERBOSE_DEFAULT;
+
+use constant _FOR_ITEM  => 0;
+use constant _FOR_ITER  => 1;
+use constant _FOR_ARRAY => 2;
+
 no warnings 'recursion';
 
 our @CARP_NOT = qw(Text::Xslate);
@@ -188,9 +199,7 @@ sub op_print_raw_s {
 sub op_include {
     my $st = Text::Xslate::PP::tx_load_template( $_[0]->self, $_[0]->{sa} );
 
-    Text::Xslate::PP::tx_execute( $st, undef, $_[0]->{vars} );
-
-    $_[0]->{ output } .= $st->{ output };
+    $_[0]->{ output } .= Text::Xslate::PP::tx_execute( $st, $_[0]->{vars} );
 
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
@@ -212,8 +221,9 @@ sub op_for_start {
 
     $_[0]->pad( $_[0]->frame->[ $_[0]->current_frame ] );
 
-    tx_access_lvar( $_[0], $id + 1, $ar );
-    tx_access_lvar( $_[0], $id + 2, -1 );
+    #tx_access_lvar( $_[0], $id + _FOR_ITEM, undef );
+    tx_access_lvar( $_[0], $id + _FOR_ITER, -1 );
+    tx_access_lvar( $_[0], $id + _FOR_ARRAY, $ar );
 
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
@@ -221,16 +231,21 @@ sub op_for_start {
 
 sub op_for_iter {
     my $id = $_[0]->{sa};
-    my $av = tx_access_lvar( $_[0], $id + 1 );
-    my $i  = tx_access_lvar( $_[0], $id + 2 );
+    my $av = tx_access_lvar( $_[0], $id + _FOR_ARRAY );
 
     if(defined $av) {
+        my $i = tx_access_lvar( $_[0], $id + _FOR_ITER );
         $av = [ $av ] unless ref $av;
-        if ( ++$i <= $#{ $av } ) {
-            tx_access_lvar( $_[0], $id     => $av->[ $i ] );
-            tx_access_lvar( $_[0], $id + 2 => $i );
+        if ( ++$i < scalar(@{ $av })  ) {
+            tx_access_lvar( $_[0], $id + _FOR_ITEM, $av->[ $i ] );
+            tx_access_lvar( $_[0], $id + _FOR_ITER, $i );
             goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
         }
+        else {
+            tx_access_lvar( $_[0], $id + _FOR_ITEM,  undef );
+            tx_access_lvar( $_[0], $id + _FOR_ITER,  undef );
+            tx_access_lvar( $_[0], $id + _FOR_ARRAY, undef );
+        }
     }
 
     # finish
@@ -346,45 +361,33 @@ sub op_minus {
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
+sub op_size {
+    $_[0]->{sa} = scalar @{ $_[0]->{sa} };
+    goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
+}
 
-sub op_eq {
-    my $aval = $_[0]->{sa};
-    my $bval = $_[0]->{sb};
-
-    if ( defined $aval and defined $bval ) {
-        # SVf_IOKかどうかのチェック
-        $_[0]->{sa} = $aval eq $bval;
+sub _sv_eq {
+    my($x, $y) = @_;
+    if ( defined $x and defined $y ) {
+        return $x eq $y;
     }
 
-    if ( defined $aval ) {
-        $_[0]->{sa} = defined $bval && $aval eq $bval;
+    if ( defined $x ) {
+        return defined $y && $x eq $y;
     }
     else {
-        $_[0]->{sa} = !defined $bval;
+        return !defined $y;
     }
+}
 
+sub op_eq {
+    $_[0]->{sa} =  _sv_eq($_[0]->{sa}, $_[0]->{sb});
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
 
 sub op_ne {
-    my $aval = $_[0]->{sa};
-    my $bval = $_[0]->{sb};
-
-    if ( defined $aval and defined $bval ) {
-        # SVf_IOKかどうかのチェック
-        $_[0]->{sa} = $aval eq $bval;
-    }
-
-    if ( defined $aval ) {
-        $_[0]->{sa} = defined $bval && $aval eq $bval;
-    }
-    else {
-        $_[0]->{sa} = !defined $bval;
-    }
-
-    $_[0]->{sa} = ! $_[0]->{sa};
-
+    $_[0]->{sa} = !_sv_eq($_[0]->{sa}, $_[0]->{sb});
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
@@ -567,32 +570,37 @@ sub tx_push_frame {
     $st->frame->[ $st->current_frame ];
 }
 
-
 sub tx_call {
-    my ( $st, $flag, $proc, @args ) = @_;
-    my $obj = shift @args if ( $flag );
+    my ( $st, $is_method_call, $proc, @args ) = @_;
     my $ret;
 
-    if ( $flag ) { # method call
+    if ( $is_method_call ) { # XXX: fetch() doesn't use methodcall for speed
+        my $obj = shift @args;
+
         unless ( defined $obj ) {
             tx_warn( $st, "Use of nil to invoke method %s", $proc );
         }
         else {
-            local $SIG{__DIE__}; # oops
-            local $SIG{__WARN__};
             $ret = eval { $obj->$proc( @args ) };
+            #_error( $st, $frame, $line, "%s\t...", $@) if $@;
         }
     }
     else { # function call
-            local $SIG{__DIE__}; # oops
-            local $SIG{__WARN__};
+        if(!defined $proc) {
+            my $c = $st->{code}->[ $st->{pc} - 1 ];
+            tx_error( $st, "Undefined function is called%s",
+                $c->{ opname } eq 'fetch_s' ? " $c->{arg}()" : ""
+            );
+        }
+        else {
             $ret = eval { $proc->( @args ) };
+            tx_error( $st, "%s\t...", $@) if $@;
+        }
     }
 
-    $ret;
+    return $ret;
 }
 
-
 sub tx_fetch {
     my ( $st, $var, $key ) = @_;
     my $ret;
